# Caching

Network data is cached at a few different levels in order to improve UI
responsiveness. There are two primary types of content that are cached:

1. REST API responses
2. Static assets (files)

## Overview

![Cache diagram](https://user-images.githubusercontent.com/1885153/28044152-c9f076ce-6629-11e7-9c6b-1bdf31fb6b50.png)

([from web platform diagrams](https://docs.google.com/presentation/d/1c8jf8UtGa81Ay4oqlbHfoP4Ile2VkiRVrCMVag6cRBQ/#slide=id.g1f921bb8da_0_0))

## 1. API responses

Currently the Meetup REST API does not provide a
[`Cache-Control` header](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Cache-Control)
(e.g. `max-age`), so the web application doesn't have any upstream indication of
how long the data should be considered valid for. The platform therefore does
not cache 'raw' API responses at the app server or browser level - the app
re-fetches every API request from the  browser to ensure that it delivers valid
data.

### CDN

The CDN does not currently cache any API responses. See the 'Future'
discussion #322 for potential changes to the behavior.

### App server

The app server does not currently cache any API responses. See the 'Future' 
discussion #322 for potential changes to the behavior.

### Browser

#### Service Worker

The service worker does not cache API requests/responses (and probably
shouldn't - see the 'Future' discussion #322).

#### Cache middleware

The cache epic (middleware) provides optimistic `state` updates for any data
specified by dispatched `GET` queries, similar to the Sync epic, but uses
locally cached data rather than an HTTP response. It is client-side only, and it
*does not suppress* data fetched from the server - it simply pre-empts it before
the API response returns and overwrites everything with the latest server data.
See the 'Future' discussion #322 for potential changes to this behavior.

The cache middleware implementation uses
[IndexedDB](https://developer.mozilla.org/en/docs/Web/API/IndexedDB_API) for
local storage of API response data - it caches the JSON response rather than the
full HTTP response in order to avoid complexity arising from the batched API
request/response routine.

##### Cache lifecycle

The cache is stored locally in the user's browser using IndexedDB so that it
will survive multiple sessions. All API queries are stored, and the data is
refreshed each time the query is re-requested. Cached data survives as long as
the browser allows the `IndexedDB` table to grow - it is theoretically
unbounded.

- **on `API_REQ`**, which provides `queries` to be sent to the API:

    1. For each query, use the JSON-encoded query as a key into the cache
    2. Format responses from the cache to look like API responses
    3. Trigger `CACHE_SUCCESS` containing the cache hits

- **on `API_SUCCESS`**,  which provides fresh info from the server

    - make an entry in the cache with `key` as the JSON-encoded `query` and
      the `value` as the corresponding `response`

- **on `CACHE_CLEAR`** (e.g. when navigating to /logout)
    
    - clear all data from cache

To manually clear the cache, go to the 'Application' tab in Chrome Dev Tools
while viewing meetup.com. 'IndexedDB' will appear in the left sidebar containing
a 'keyval-store' table - click on that and then click on 'Delete database' in
order to clear the cache. The database will be re-created next time you open a
web platform-based page.

In dev, be aware that the cache may be masking failed API requests - keep your
network dev tools open and watch the server logs to make sure you haven't broken
anything.

## 2. Static assets

Static assets generated by the build are cached in our CDN and in the browser.
Because each filename includes a hash of the file contents, the file can have an
infinite cache lifetime.

### CDN

All static assets are delivered to the browser with a `Cache-Control` header
with a a 10 year expiration.

### Browser

#### Service Worker pre-cache

For browsers that support service workers, the platform service worker will 
'pre-cache' all bundled assets asynchronously after the in-page assets load, and
serve those assets from the browser cache for all future requests.

**Timeline**

1. First webpage request
2. rendered page HTML downloads
3. current static assets download (CSS, JS)
4. JS executes, registers service worker
5. Service worker immediately fetches other static assets that are used
   elsewhere on the site, e.g. async routes and static images/icons.

When new assets are generated, a new service worker is generated. When a client
loads the site, the new service worker will replace any existing service workers
and immediately start downloading new static assets. Any un-changed static
assets (hashed filename unchanged) will not be re-downloaded.

#### No Service Worker support

For
[browsers that do not support Service Workers](https://jakearchibald.github.io/isserviceworkerready/),
normal browser cache rules apply - static assets will be cached as they are
requested by the app, but not before. 
