'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.cacheQueryEpic = exports.cacheSetEpic = exports.cacheClearEpic = undefined;
exports.checkEnable = checkEnable;

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/from');

require('rxjs/add/operator/zip');

require('rxjs/add/operator/map');

require('rxjs/add/operator/filter');

require('rxjs/add/operator/reduce');

require('rxjs/add/operator/ignoreElements');

require('rxjs/add/operator/mergeMap');

var _reduxObservable = require('redux-observable');

var _apiActionCreators = require('../sync/apiActionCreators');

var _cacheActionCreators = require('./cacheActionCreators');

var _util = require('./util');

/**
 * Provides a cache outside of Redux state that can optimistically update state
 * before an asynchronous API call returns
 *
 * @module CacheMiddleware
 */
function checkEnable() {
	if (typeof window !== 'undefined' && window.location) {
		var currentUrl = new URL(window.location.href);
		return !currentUrl.searchParams.has('__nocache');
	}
	return true;
}

/**
 * Listen for any action that should clear cached state
 *
 * Note that this will clear the cache without emitting an action
 */
var cacheClearEpic = exports.cacheClearEpic = function cacheClearEpic(cache) {
	return function (action$) {
		return action$.ofType(_cacheActionCreators.CACHE_CLEAR).mergeMap(function () {
			return cache.clear();
		}) // wait for cache to clear before continuing
		.ignoreElements();
	};
};

/**
 * Listen for any action that should set cached state with a
 * `{ queries, responses }` payload
 *
 * API_SUCCESS means there is fresh data ready to be stored - extract the
 * queries and their responses, then dispatch `CACHE_SET` actions with each
 * pair
 *
 * Not that this will set the cache without emitting an action
 */
var cacheSetEpic = exports.cacheSetEpic = function cacheSetEpic(cache) {
	return function (action$) {
		return action$.ofType(_apiActionCreators.API_RESP_SUCCESS, _cacheActionCreators.CACHE_SET).mergeMap(function (_ref) {
			var _ref$payload = _ref.payload,
			    query = _ref$payload.query,
			    response = _ref$payload.response;
			return (0, _util.cacheWriter)(cache)(query, response);
		}).ignoreElements();
	};
};

/**
 * Listen for any action that should query the cache using a payload of queries
 *
 * Observables are heavily used in CACHE_REQUEST because each query results in
 * an async 'get' (Promise) from the Cache - all 'gets' happen in parallel and
 * the results are collated into a single response object containing the cache
 * hits.
 */
var cacheQueryEpic = exports.cacheQueryEpic = function cacheQueryEpic(cache) {
	return function (action$) {
		return action$.ofType(_apiActionCreators.API_REQ).mergeMap(function (_ref2) {
			var payload = _ref2.payload;
			return _Observable.Observable.from(payload) // fan out
			.mergeMap((0, _util.cacheReader)(cache)) // look for a cache hit
			.filter(function (_ref3) {
				var query = _ref3.query,
				    response = _ref3.response;
				return response;
			});
		} // ignore misses
		).map(_cacheActionCreators.cacheSuccess);
	};
};

var getCacheEpic = function getCacheEpic() {
	var cache = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _util.makeCache)();
	return checkEnable() ? (0, _reduxObservable.combineEpics)(cacheClearEpic(cache), cacheSetEpic(cache), cacheQueryEpic(cache)) : function (action$) {
		return action$.ignoreElements();
	};
};

exports.default = getCacheEpic;
//# sourceMappingURL=index.js.map