'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.makeCache = makeCache;
function makeCache() {
	if (typeof window === 'undefined' || !window.indexedDB) {
		var _data = {};
		return {
			get: function get(key) {
				return Promise.resolve(_data[key]);
			},
			set: function set(key, val) {
				_data[key] = val;
				return Promise.resolve(true);
			},
			delete: function _delete(key) {
				delete _data[key];
				return Promise.resolve(true);
			},
			clear: function clear() {
				Object.keys(_data).forEach(function (key) {
					return delete _data[key];
				});
				return Promise.resolve(true);
			}
		};
	}

	return require('idb-keyval');
}

/**
 * Generates a function that can read queries and return hits in the supplied cache
 *
 * @param {Object} cache the persistent cache containing query-able data
 * @param {Object} query query for app data
 * @return {Promise} resolves with cache hit, otherwise rejects
 */

/**
 * This function performs feature sniffing to determine whether the preferred
 * IndexedDB cache is available, otherwise it falls back to a simple
 * plain-object-based cache that will only survive as long as the request.
 *
 * The cache object methods are thin wrappers around their IndexedDB
 * ObjectStore equivalents
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/API/IDBObjectStore}
 *
 * @returns {Object} an object with Promise-based `get`, `set`, `delete`, and
 * `clear` methods
 */
var cacheReader = exports.cacheReader = function cacheReader(cache) {
	return function (query) {
		return cache.get(JSON.stringify(query)).then(function (response) {
			return { query: query, response: response };
		}).catch(function (err) {
			return { query: query, response: null };
		});
	};
}; // errors don't matter - just return null

/**
 * Generates a function that can write query-response values into cache
 *
 * @param {Object} cache the persistent cache containing query-able data
 * @param {Object} query query for app data
 * @param {Object} response plain object API response for the query
 * @return {Promise}
 */
var cacheWriter = exports.cacheWriter = function cacheWriter(cache) {
	return function (query, response) {
		var method = (query.meta || {}).method || 'get';
		if (method.toLowerCase() !== 'get') {
			return Promise.resolve(true);
		}
		return cache.set(JSON.stringify(query), response);
	};
};
//# sourceMappingURL=util.js.map