'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.requestAll = requestAll;
exports.success = success;
exports.error = error;
exports.fail = fail;
exports.complete = complete;
var API_REQ = exports.API_REQ = 'API_REQ';
var API_RESP_SUCCESS = exports.API_RESP_SUCCESS = 'API_RESP_SUCCESS';
var API_RESP_COMPLETE = exports.API_RESP_COMPLETE = 'API_RESP_COMPLETE';
var API_RESP_ERROR = exports.API_RESP_ERROR = 'API_RESP_ERROR';
var API_RESP_FAIL = exports.API_RESP_FAIL = 'API_RESP_FAIL';

/**
 * Generic base action creator upon which all individual query action creators
 * are built. This action creator is designed to support parallel queries with
 * request metadata, which primarily appears in navigation-related actions.
 *
 * All request actions will get a Promise object injected into the `meta`
 * property, along with `resolve` and `reject` methods that can be used to
 * programmatically resolve the Promise. The expectation is that this Promise
 * will be used to indicate the state of the request, and will resolve when the
 * request is resolved or rejected.
 *
 * @param {Array} queries the queries to send to the server
 * @param {Object} meta metadata about the request, e.g. 'logout', 'clickTracking'
 * @return {Object} an API_REQ action
 */
function requestAll(queries, meta) {
	meta = meta || {};
	meta.request = new Promise(function (resolve, reject) {
		meta = meta || {};
		meta.resolve = resolve;
		meta.reject = reject;
	});
	return {
		type: API_REQ,
		payload: queries,
		meta: meta
	};
}

var _applyMethod = function _applyMethod(method) {
	return function (query, meta) {
		query.meta = _extends({}, query.meta || {}, {
			method: method
		});
		// delegate to `requestAll`
		var requestAction = requestAll([query], meta);
		// modify the promise to automatically pull out the single response
		// corresponding to the single query
		requestAction.meta.promise = requestAction.meta.request.then(function (responses) {
			return responses[0];
		});
		return requestAction;
	};
};

var get = exports.get = _applyMethod('get');
var post = exports.post = _applyMethod('post');
var patch = exports.patch = _applyMethod('patch');
var del = exports.del = _applyMethod('delete');

function success(_ref) {
	var query = _ref.query,
	    response = _ref.response;

	return {
		type: API_RESP_SUCCESS,
		payload: { query: query, response: response }
	};
}

function error(_ref2) {
	var query = _ref2.query,
	    response = _ref2.response;

	return {
		type: API_RESP_ERROR,
		payload: { query: query, response: response }
	};
}

function fail(err) {
	return {
		type: API_RESP_FAIL,
		payload: err
	};
}

function complete(queries) {
	return {
		type: API_RESP_COMPLETE,
		payload: queries
	};
}
//# sourceMappingURL=apiActionCreators.js.map