'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.checkForDevUrl = checkForDevUrl;
exports.onRequestExtension = onRequestExtension;
exports.logResponse = logResponse;
exports.registerExtensionEvents = registerExtensionEvents;
exports.configureEnv = configureEnv;
exports.server = server;

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _hapi = require('hapi');

var _hapi2 = _interopRequireDefault(_hapi);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * determine whether a nested object of values has
 * a string that contains `.dev.meetup.`
 *
 * @param {String|Object} value string or nested object
 * with values that could be URL strings
 *
 * @return {Boolean} whether the `value` contains a 'dev' URL string
 */
function checkForDevUrl(value) {
	switch (typeof value === 'undefined' ? 'undefined' : _typeof(value)) {
		case 'string':
			return value.indexOf('.dev.meetup.') > -1;
		case 'object':
			return Object.keys(value).some(function (key) {
				return checkForDevUrl(value[key]);
			});
	}

	return false;
}

function onRequestExtension(request, reply) {
	request.id = _uuid2.default.v4();

	request.server.app.logger.info('Incoming request ' + request.method.toUpperCase() + ' ' + request.url.href);

	return reply.continue();
}

function logResponse(request) {
	var method = request.method,
	    response = request.response,
	    id = request.id,
	    info = request.info,
	    logger = request.server.app.logger,
	    url = request.url;


	if (response.isBoom) {
		// response is an Error object
		logger.error('Internal error ' + response.message + ' ' + url.pathname, {
			error: response.stack
		});
	}

	var log = (response.statusCode >= 400 && logger.error || response.statusCode >= 300 && logger.warn || logger.info).bind(logger);

	log({
		headers: response.headers,
		id: id,
		method: method,
		referrer: info.referrer,
		remoteAddress: info.remoteAddress,
		time: info.responded - info.received,
		href: url.href
	}, 'Outgoing response ' + method.toUpperCase() + ' ' + url.pathname + ' ' + response.statusCode);

	return;
}

/**
 * Use server.ext to add functions to request/server extension points
 * @see {@link https://hapijs.com/api#request-lifecycle}
 * @param {Object} server Hapi server
 * @return {Object} Hapi server
 */
function registerExtensionEvents(server) {
	server.ext([{
		type: 'onRequest',
		method: onRequestExtension
	}]);
	server.on('response', logResponse);
	return server;
}

/**
 * Make any environment changes that need to be made in response to the provided
 * config
 *
 * @param {Object} config the environment configuration object
 *
 * @return null
 */
function configureEnv(config) {
	// When using .dev.meetup endpoints, ignore self-signed SSL cert
	var USING_DEV_ENDPOINTS = checkForDevUrl(config);

	_https2.default.globalAgent.options.rejectUnauthorized = !USING_DEV_ENDPOINTS;
}

/**
 * server-starting function
 */
function server(routes, connection, plugins, config) {
	var server = new _hapi2.default.Server();

	// store runtime state - must modify existing server.settings.app in order to keep
	// previously-defined properties
	// https://hapijs.com/api#serverapp
	server.settings.app = Object.assign(server.settings.app || {}, config);

	var appConnection = server.connection(connection);
	return appConnection.register(plugins).then(function () {
		return registerExtensionEvents(server);
	}).then(function () {
		return server.auth.strategy('default', 'oauth', true);
	}).then(function () {
		return appConnection.route(routes);
	}).then(function () {
		return server.start();
	}).then(function () {
		return server;
	});
}
//# sourceMappingURL=index.js.map