'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.makeReceive = exports.makeInjectResponseCookies = exports.makeLogResponse = exports.makeApiResponseToQueryResponse = exports.makeParseApiResponse = exports.parseApiValue = exports.parseMetaHeaders = exports.parseVariantsHeader = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _2 = require('../');

var _stringUtils = require('./stringUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// match escpaed unicode characters that are treated as newline literals in JS
var ESCAPED_UNICODE_NEWLINES = /\\u2028|\\u2029/g;

/**
 * Convert the X-Meetup-Variants response header into a state-ready object
 *
 * @see {@link https://meetup.atlassian.net/wiki/display/MUP/X-Meetup-Variants}
 * @returns {Object} {
 *   [experiment]: {
 *     [context (member/chapter id)]: variantName
 *   }
 * }
 */
var parseVariantsHeader = exports.parseVariantsHeader = function parseVariantsHeader(variantsHeader) {
	return variantsHeader.split(' ').reduce(function (variants, keyval) {
		var _keyval$split = keyval.split('='),
		    _keyval$split2 = _slicedToArray(_keyval$split, 2),
		    experiment = _keyval$split2[0],
		    val = _keyval$split2[1];

		variants[experiment] = variants[experiment] || {};

		var _val$split = val.split('|'),
		    _val$split2 = _slicedToArray(_val$split, 2),
		    context = _val$split2[0],
		    variant = _val$split2[1];

		variants[experiment][context] = variant || null;
		return variants;
	}, {});
};

var X_HEADERS = ['x-total-count'];

var parseMetaHeaders = exports.parseMetaHeaders = function parseMetaHeaders(headers) {
	var meetupHeaders = Object.keys(headers).filter(function (h) {
		return h.startsWith('x-meetup-');
	}).reduce(function (meta, h) {
		var key = (0, _stringUtils.toCamelCase)(h.replace('x-meetup-', ''));
		meta[key] = headers[h];
		return meta;
	}, {});

	// special case handling for flags
	if (meetupHeaders.flags) {
		meetupHeaders.flags = _qs2.default.parse(meetupHeaders.flags, {
			delimiter: ', ',
			decoder: _stringUtils.coerceBool
		});
	}

	// special case handling for variants
	if (meetupHeaders.variants) {
		meetupHeaders.variants = parseVariantsHeader(meetupHeaders.variants);
	}

	var xHeaders = X_HEADERS.reduce(function (meta, h) {
		var key = (0, _stringUtils.toCamelCase)(h.replace('x-', ''));
		if (h in headers) {
			meta[key] = headers[h];
		}
		return meta;
	}, {});

	var linkHeader = headers.link && headers.link.split(',').reduce(function (links, link) {
		var _link$split = link.split(';'),
		    _link$split2 = _slicedToArray(_link$split, 2),
		    urlString = _link$split2[0],
		    relString = _link$split2[1];

		var url = urlString.replace(/<|>/g, '').trim();
		var rel = relString.replace(/rel="([^"]+)"/, '$1').trim();
		links[rel] = url;
		return links;
	}, {});
	var meta = _extends({}, meetupHeaders, xHeaders);

	if (linkHeader) {
		meta.link = linkHeader;
	}
	return meta;
};

/**
 * Accept an Error and return an object that will be used in place of the
 * expected API return value
 * @param {Error} err the error to populate
 * @return {Object} { value, error? }
 */
function formatApiError(err) {
	var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

	return {
		value: value,
		error: err.message
	};
}

var parseBody = function parseBody(body) {
	if (!body) {
		return null;
	}
	// Some newline literals will not work in JS string literals - they must be
	// converted to an escaped newline character that will work end to end ('\n')
	// treat non-success HTTP code as an error
	var safeBody = body.replace(ESCAPED_UNICODE_NEWLINES, '\\n');
	return JSON.parse(safeBody);
};

/**
 * The externalRequest callback provides a `response` object and a `body` string
 * that need to be parsed in order to determine the appropriate 'value'.
 * 
 * This function determines the { value, error } based on the status code and
 * body of the response - it will always set an 'error' value when a non-2xx
 * response is received, but it may provide additional error details that are
 * included in the 'body' of the response - body error details will usually
 * be JSON parsed into `value.errors`, but that is determined by the JSON
 * returned by the REST API.
 * 
 * @return {Object} { value, error? }
 */
var parseApiValue = exports.parseApiValue = function parseApiValue(_ref) {
	var _ref2 = _slicedToArray(_ref, 2),
	    response = _ref2[0],
	    body = _ref2[1];

	if (response.statusCode === 204) {
		// NoContent response type
		return { value: null };
	}

	try {
		var value = parseBody(body);
		if (response.statusCode < 200 || response.statusCode >= 400) {
			return formatApiError(new Error(response.statusMessage), value);
		}
		if (value && value.problem) {
			return formatApiError(new Error('API problem: ' + value.problem + ': ' + value.details), value);
		}
		return { value: value };
	} catch (err) {
		return formatApiError(err);
	}
};

/**
 *
 * mostly error handling - any case where the API does not satisfy the
 * "api response" formatting requirement: plain object containing the requested
 * values
 *
 * This utility is specific to the response format of the API being consumed
 * @param {Array} the callback args for npm request - [response, body], where
 * `response` is an `Http.IncomingMessage` and `body` is the body text of the
 * response.
 * @return responseObj the JSON-parsed text, possibly with error info
 */
var makeParseApiResponse = exports.makeParseApiResponse = function makeParseApiResponse(query) {
	return function (_ref3) {
		var _ref4 = _slicedToArray(_ref3, 2),
		    response = _ref4[0],
		    body = _ref4[1];

		var _parseApiValue = parseApiValue([response, body]),
		    value = _parseApiValue.value,
		    error = _parseApiValue.error;

		var meta = _extends({}, parseMetaHeaders(response.headers), {
			endpoint: query.endpoint,
			statusCode: response.statusCode
		});

		return {
			value: value,
			error: error,
			meta: meta
		};
	};
};
/**
 * Format apiResponse to match expected state structure
 *
 * @param {Object} apiResponse JSON-parsed api response data
 */
var makeApiResponseToQueryResponse = exports.makeApiResponseToQueryResponse = function makeApiResponseToQueryResponse(query) {
	return function (_ref5) {
		var value = _ref5.value,
		    error = _ref5.error,
		    meta = _ref5.meta;
		return {
			type: query.type,
			ref: query.ref,
			value: value,
			error: error,
			meta: meta
		};
	};
};

var makeLogResponse = exports.makeLogResponse = function makeLogResponse(request) {
	return function (_ref6) {
		var _ref7 = _slicedToArray(_ref6, 2),
		    response = _ref7[0],
		    body = _ref7[1];

		var elapsedTime = response.elapsedTime,
		    _response$request = response.request,
		    id = _response$request.id,
		    _response$request$uri = _response$request.uri,
		    query = _response$request$uri.query,
		    pathname = _response$request$uri.pathname,
		    href = _response$request$uri.href,
		    method = _response$request.method,
		    statusCode = response.statusCode;

		var parsedQuery = (query || '').split('&').reduce(function (acc, keyval) {
			var _keyval$split3 = keyval.split('='),
			    _keyval$split4 = _slicedToArray(_keyval$split3, 2),
			    key = _keyval$split4[0],
			    val = _keyval$split4[1];

			acc[key] = val;
			return acc;
		}, {});

		if (statusCode >= 500 || // REST API had an internal error
		method.toLowerCase() === 'get' && statusCode >= 400 // something fishy with a GET
		) {
				// use console.error or console.warn to highlight these cases in Stackdriver
				// Only want to warn about 400s for now (maybe forever?)
				var logError = statusCode < 500 ? console.warn : console.error;
				logError(JSON.stringify({
					message: 'REST API error response',
					info: {
						url: href,
						query: parsedQuery,
						method: method,
						id: id,
						statusCode: statusCode,
						time: elapsedTime,
						originRequestId: request.id,
						body: body
					}
				}));
			}
		var logger = request.server.app.logger;
		// production logs will automatically be JSON-parsed in Stackdriver
		var log = (statusCode >= 500 && logger.error || statusCode >= 400 && logger.warn || logger.info).bind(logger);

		log({
			type: 'response',
			direction: 'in',
			info: {
				url: href,
				query: parsedQuery,
				method: method,
				id: id,
				originRequestId: request.id,
				statusCode: statusCode,
				time: elapsedTime,
				body: body.length > 256 ? body.substr(0, 256) + '...' : body
			}
		}, 'Incoming response ' + method.toUpperCase() + ' ' + pathname + ' ' + response.statusCode);
	};
};

/**
 * When a tough-cookie cookie jar is provided, forward the cookies along with
 * the overall /api response back to the client
 */
var makeInjectResponseCookies = exports.makeInjectResponseCookies = function makeInjectResponseCookies(request) {
	return function (_ref8) {
		var _ref9 = _slicedToArray(_ref8, 3),
		    response = _ref9[0],
		    _ = _ref9[1],
		    jar = _ref9[2];

		if (!jar) {
			return;
		}
		var requestUrl = response.toJSON().request.uri.href;
		jar.getCookies(requestUrl).forEach(function (cookie) {
			var cookieOptions = {
				domain: cookie.domain,
				path: cookie.path,
				isHttpOnly: cookie.httpOnly,
				isSameSite: false,
				isSecure: request.server.settings.app.isProd,
				strictHeader: false // Can't enforce RFC 6265 cookie validation on external services
			};

			request.plugins[_2.API_PROXY_PLUGIN_NAME].setState(cookie.key, cookie.value, cookieOptions);
		});
	};
};

var makeReceive = exports.makeReceive = function makeReceive(request) {
	var logResponse = makeLogResponse(request);
	var injectResponseCookies = makeInjectResponseCookies(request);
	return function (query) {
		var parseApiResponse = makeParseApiResponse(query);
		var apiResponseToQueryResponse = makeApiResponseToQueryResponse(query);
		return function (response) {
			logResponse(response); // this will leak private API response data into production logs
			injectResponseCookies(response);
			try {
				return apiResponseToQueryResponse(parseApiResponse(response));
			} catch (err) {
				return _extends({}, formatApiError(err), {
					meta: {
						endpoint: query.endpoint
					}
				});
			}
		};
	};
};
//# sourceMappingURL=receive.js.map