'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.oauthScheme = exports.getAuthenticate = exports.getRequestAuthorizer$ = exports.getAccessToken$ = exports.applyRequestAuthorizer$ = exports.tryJSON = undefined;
exports.getAnonymousCode$ = getAnonymousCode$;
exports.default = register;

var _Observable = require('rxjs/Observable');

require('rxjs/add/observable/of');

require('rxjs/add/observable/if');

require('rxjs/add/observable/defer');

require('rxjs/add/observable/fromPromise');

require('rxjs/add/operator/do');

require('rxjs/add/operator/timeout');

require('rxjs/add/operator/mergeMap');

require('rxjs/add/operator/map');

var _logger = require('../util/logger');

var _logger2 = _interopRequireDefault(_logger);

var _cookieUtils = require('../util/cookieUtils');

var _authUtils = require('../util/authUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @module requestAuthPlugin
 */

/**
 * Attempt to JSON parse a Response object from a fetch call
 *
 * @param {String} reqUrl the URL that was requested
 * @param {Response} response the fetch Response object
 * @return {Promise} a Promise that resolves with the JSON-parsed text
 */
var tryJSON = exports.tryJSON = function tryJSON(reqUrl) {
	return function (response) {
		var status = response.status,
		    statusText = response.statusText;

		if (status >= 400) {
			// status always 200: bugzilla #52128
			return Promise.reject(new Error('Request to ' + reqUrl + ' responded with error code ' + status + ': ' + statusText));
		}
		return response.text().then(function (text) {
			return JSON.parse(text);
		});
	};
};

var verifyAuth = function verifyAuth(logger) {
	return function (auth) {
		if (!Object.keys(auth).length) {
			var errorMessage = 'No auth token(s) provided';
			logger.fatal(errorMessage, ': application can not fetch data.', 'You might be able to recover by clearing cookies and refreshing');
			throw new Error(errorMessage);
		}
	};
};

function getAuthType(request) {
	var allowedAuthTypes = [_cookieUtils.MEMBER_COOKIE, 'oauth_token'];
	// search for a request.state cookie name that matches an allowed auth type
	return allowedAuthTypes.reduce(function (authType, allowedType) {
		return authType || request.state[allowedType] && allowedType;
	}, null);
}

/**
 * Ensure that the passed-in Request contains a valid Oauth token
 *
 * If the Request already has a valid oauth token, it is returned unchanged,
 * otherwise the request is parsed for more info and a new token is set
 *
 * @param {Observable} requestAuthorizer$ a function that takes a request and emits new auth
 *   data
 * @param {Request} request Hapi request to modify with auth token (if necessary)
 * @return {Observable} Observable that emits the request with auth applied
 */
var applyRequestAuthorizer$ = exports.applyRequestAuthorizer$ = function applyRequestAuthorizer$(requestAuthorizer$) {
	return function (request) {
		var plugins = request.plugins,
		    logger = request.server.app.logger,
		    req = request.raw.req;

		// always need oauth_token, even if it's an anonymous (pre-reg) token
		// This is 'deferred' because we don't want to start fetching the token
		// before we know that it's needed

		var authType = getAuthType(request);

		if (authType) {
			plugins.requestAuth.authType = authType;
			return _Observable.Observable.of(request);
		}

		logger.warn({ req: req }, 'Request does not contain auth token');
		return requestAuthorizer$(request) // get anonymous oauth_token
		.do((0, _authUtils.applyAuthState)(request, plugins.requestAuth.reply)).map(function () {
			return request;
		});
	};
};

/**
 * Get an anonymous code from the API that can be used to generate an oauth
 * access token
 *
 * @param {Object} serverAppConfig the Hapi `server.settings.app` object
 * @param {String} redirect_uri Return url after anonymous grant
 */
function getAnonymousCode$(serverAppConfig, redirect_uri) {
	if (!serverAppConfig.oauth.key) {
		throw new ReferenceError('OAuth consumer key is required');
	}

	var authURL = new URL(serverAppConfig.oauth.auth_url);
	authURL.searchParams.append('response_type', 'anonymous_code');
	authURL.searchParams.append('client_id', serverAppConfig.oauth.key);
	authURL.searchParams.append('redirect_uri', redirect_uri);
	var requestOpts = {
		method: 'GET',
		headers: {
			Accept: 'application/json'
		}
	};

	return _Observable.Observable.defer(function () {
		_logger2.default.info({
			type: 'request',
			direction: 'out',
			info: {
				url: serverAppConfig.oauth.auth_url,
				method: 'get'
			}
		}, 'Outgoing request GET ' + serverAppConfig.oauth.auth_url);

		var startTime = new Date();
		return _Observable.Observable.fromPromise(fetch(authURL.toString(), requestOpts)).timeout(serverAppConfig.api.timeout).do(function () {
			_logger2.default.info({
				type: 'response',
				direction: 'in',
				info: {
					url: serverAppConfig.oauth.auth_url,
					method: 'get',
					responseTime: new Date() - startTime
				}
			}, 'Incoming response GET ' + serverAppConfig.oauth.auth_url);
		}).mergeMap(tryJSON(serverAppConfig.oauth.auth_url)).map(function (_ref) {
			var code = _ref.code;
			return {
				grant_type: 'anonymous_code',
				token: code
			};
		});
	});
}

/**
 * Generate a function that receives a grant type and grant
 * token that can be used to generate an oauth access token from the API
 *
 * @param {Object} serverAppConfig the Hapi `server.settings.app` config object
 * @param {String} redirect_uri Return url after anonymous grant
 *
 * @return {Object} the JSON-parsed response from the authorize endpoint
 *   - contains 'access_token', 'refresh_token'
 */
var getAccessToken$ = exports.getAccessToken$ = function getAccessToken$(serverAppConfig, redirect_uri) {
	if (!serverAppConfig.oauth.key) {
		throw new ReferenceError('OAuth consumer key is required');
	}
	if (!serverAppConfig.oauth.secret) {
		throw new ReferenceError('OAuth consumer secret is required');
	}
	var params = {
		client_id: serverAppConfig.oauth.key,
		client_secret: serverAppConfig.oauth.secret,
		redirect_uri: redirect_uri
	};
	return function (headers) {
		var requestOpts = {
			method: 'POST',
			headers: {
				Cookie: headers.cookie,
				Accept: headers.accept,
				'Accept-Language': headers['accept-language'],
				'Cache-Control': headers['cache-control']
			}
		};
		var accessUrl = Object.keys(params).reduce(function (accessUrl, key) {
			accessUrl.searchParams.append(key, params[key]);
			return accessUrl;
		}, new URL(serverAppConfig.oauth.access_url));

		return function (_ref2) {
			var grant_type = _ref2.grant_type,
			    token = _ref2.token;

			if (!token) {
				// programmer error or catastrophic auth failure - throw exception
				throw new ReferenceError('No grant token provided - cannot obtain access token');
			}

			accessUrl.searchParams.append('grant_type', grant_type);
			if (grant_type === 'anonymous_code') {
				accessUrl.searchParams.append('code', token);
			}
			if (grant_type === 'refresh_token') {
				accessUrl.searchParams.append('refresh_token', token);
			}

			_logger2.default.info({
				type: 'request',
				direction: 'out',
				info: {
					url: serverAppConfig.oauth.access_url,
					method: 'get'
				}
			}, 'Outgoing request GET ' + serverAppConfig.oauth.access_url + '?' + grant_type);

			var startTime = new Date();
			return _Observable.Observable.fromPromise(fetch(accessUrl.toString(), requestOpts)).timeout(serverAppConfig.api.timeout).do(function () {
				_logger2.default.info({
					type: 'response',
					direction: 'in',
					info: {
						url: serverAppConfig.oauth.access_url,
						method: 'get',
						responseTime: new Date() - startTime
					}
				}, 'Incoming response GET ' + serverAppConfig.oauth.access_url + '?' + grant_type);
			}).mergeMap(tryJSON(serverAppConfig.oauth.access_url));
		};
	};
};

var refreshToken$ = function refreshToken$(refresh_token) {
	return _Observable.Observable.of({
		grant_type: 'refresh_token',
		token: refresh_token
	});
};

/**
 * Curry a function that will get a new auth token for a passed-in request.
 * For an anonymous auth, the request header information is used to determine
 * the location and language of the anonymous member
 *
 * @param {Object} serverAppConfig the Hapi `server.settings.app` config object
 */
var getRequestAuthorizer$ = exports.getRequestAuthorizer$ = function getRequestAuthorizer$(serverAppConfig) {
	var redirect_uri = 'http://www.meetup.com/'; // required param set in oauth consumer config
	var anonymousCode$ = getAnonymousCode$(serverAppConfig, redirect_uri);
	var accessToken$ = getAccessToken$(serverAppConfig, redirect_uri);

	// if the request has a refresh_token, use it. Otherwise, get a new anonymous access token
	return function (_ref3) {
		var headers = _ref3.headers,
		    refresh_token = _ref3.state.refresh_token,
		    logger = _ref3.server.app.logger;
		return _Observable.Observable.if(function () {
			return refresh_token;
		}, refreshToken$(refresh_token), anonymousCode$).mergeMap(accessToken$(headers)).do(verifyAuth(logger));
	};
};

var getAuthenticate = exports.getAuthenticate = function getAuthenticate(authorizeRequest$) {
	return function (request, reply) {
		var logger = request.server.app.logger,
		    req = request.raw.req;

		logger.debug({ req: req }, 'Authenticating request');
		return authorizeRequest$(request).do(function (request) {
			logger.debug({ req: req }, 'Request authenticated');
		}).subscribe(function (request) {
			var credentials = request.state[getAuthType(request)] || request.plugins.requestAuth.oauth_token;
			reply.continue({ credentials: credentials, artifacts: credentials });
		}, function (err) {
			return reply(err, null, { credentials: null });
		});
	};
};

/**
 * Request authorizing scheme
 *
 * 1. assign a reference to the reply interface on request.plugins.requestAuth
 * 2. make sure the correct MEETUP_MEMBER[_DEV] cookie is used for auth
 * 3. return the authentication function from getAuthenticate, which ensures
 * that all requests have valid auth credentials (anonymous or logged in)
 *
 * https://hapijs.com/api#serverauthschemename-scheme
 * https://hapijs.com/api#serverauthstrategyname-scheme-mode-options
 *
 * @param {Object} server the Hapi app server instance
 * @param {Object} options the options passed to `server.auth.strategy`for the
 *   auth stategy instance
 */
var oauthScheme = exports.oauthScheme = function oauthScheme(server) {
	// apply default config for auth cookies
	(0, _authUtils.configureAuthCookies)(server);

	// provide a reference to `reply` on the request
	server.ext('onPreAuth', _authUtils.setPluginState);

	var authorizeRequest$ = applyRequestAuthorizer$(getRequestAuthorizer$(server.settings.app));

	return {
		authenticate: getAuthenticate(authorizeRequest$)
	};
};

/**
 * This plugin does two things.
 *
 * 1. Adds an 'authorize' interface on the Hapi `request`, which ensures that
 * the request has an oauth_token cookie - it provides an anonymous token when
 * none is provided in the request, and refreshes a token that has expired
 * 2. Adds a new route that returns the auth JSON containing the new oauth_token
 * (configurable, defaults to '/auth')
 *
 * {@link http://hapijs.com/tutorials/plugins}
 */
function register(server, options, next) {
	// register the plugin's auth scheme
	server.auth.scheme('oauth', oauthScheme);

	next();
}

register.attributes = {
	name: 'requestAuth',
	version: '1.0.0'
};
//# sourceMappingURL=requestAuthPlugin.js.map