'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getLogger = exports.ACTIVITY_PLUGIN_NAME = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.getTrackers = getTrackers;
exports.default = register;

var _avro = require('./util/avro');

var _avro2 = _interopRequireDefault(_avro);

var _activityTrackers = require('./_activityTrackers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var YEAR_IN_MS = 1000 * 60 * 60 * 24 * 365;
var ACTIVITY_PLUGIN_NAME = exports.ACTIVITY_PLUGIN_NAME = 'tracking';

/*
 * This plugin provides `request.track...` methods that track events related to
 * particular server responses, e.g. new sesssions and navigation activity.
 *
 * Available trackers:
 * - `trackActivity`
 */

var getLogger = exports.getLogger = function getLogger(agent) {
	return function (request, trackInfo) {
		var requestHeaders = request.headers;

		// Takes in desired time to convert and applies offset
		var offset = new Date().getTimezoneOffset() * 60000; // gets detected timezone + converts to milliseconds
		var nowUTC = new Date(Date.now() + offset);

		// generates new date object taking the time in milliseconds and
		// adds the runtime environment's timezone offset
		var NY_OFFSET = -14400000;
		var newNow = new Date(nowUTC.getTime() - NY_OFFSET);

		var record = _extends({
			timestamp: newNow.toISOString(),
			requestId: request.id,
			ip: requestHeaders['remote-addr'] || '',
			agent: requestHeaders['user-agent'] || '',
			platform: 'WEB',
			platformAgent: agent,
			mobileWeb: false,
			referer: '', // misspelled to align with schema
			trax: {},
			isUserActivity: true
		}, trackInfo);

		_avro2.default.loggers.activity(record);
		return record;
	};
};

/*
 * Each tracking function is a composition of a logging function and data about
 * the `request` object. This function computes some configuration information
 * to create the tracking functions, and returns each of them in a map keyed by
 * the target `request` method name
 */
function getTrackers(options) {
	var agent = options.agent,
	    browserIdCookieName = options.browserIdCookieName,
	    memberCookieName = options.memberCookieName,
	    trackIdCookieName = options.trackIdCookieName;


	var log = getLogger(agent);
	var trackOpts = {
		log: log,
		browserIdCookieName: browserIdCookieName,
		memberCookieName: memberCookieName,
		trackIdCookieName: trackIdCookieName
	};
	// These are the tracking methods that will be set on the `request` interface
	var trackers = {
		trackActivity: (0, _activityTrackers.getTrackActivity)(trackOpts),
		trackApiResponses: (0, _activityTrackers.getTrackApiResponses)(trackOpts)
	};
	return trackers;
}

/*
 * Run request-initialization routines, e.g. creating plugin data store
 */
var onRequest = function onRequest(request, reply) {
	// initialize request.plugins[ACTIVITY_PLUGIN_NAME] to store cookie vals
	request.plugins[ACTIVITY_PLUGIN_NAME] = {};
	reply.continue();
};
/*
 * Read from request data to prepare/modify response. Mainly looking for new
 * tracking cookies that need to be set using request.response.state.
 */
var getOnPreResponse = function getOnPreResponse(cookieConfig) {
	return function (request, reply) {
		var browserIdCookieName = cookieConfig.browserIdCookieName,
		    trackIdCookieName = cookieConfig.trackIdCookieName;

		var pluginData = request.plugins[ACTIVITY_PLUGIN_NAME];
		var browserId = pluginData.browserIdCookieName;
		var trackId = pluginData.trackIdCookieName;

		var FOREVER = {
			encoding: 'none',
			path: '/',
			isHttpOnly: true,
			ttl: YEAR_IN_MS * 20
		};

		if (browserId) {
			request.response.state(browserIdCookieName, browserId, FOREVER);
		}
		if (trackId) {
			request.response.state(trackIdCookieName, trackId, FOREVER);
		}
		reply.continue();
	};
};

/*
 * The plugin register function that will 'decorate' the `request` interface with
 * all tracking functions returned from `getTrackers`, as well as assign request
 * lifecycle event handlers that can affect the response, e.g. by setting cookies
 */
function register(server, options, next) {
	var agent = options.agent,
	    isProd = options.isProd;


	var trackIdCookieName = isProd ? 'MEETUP_TRACK' : 'MEETUP_TRACK_DEV';
	var browserIdCookieName = isProd ? 'MEETUP_BROWSER_ID' : 'MEETUP_BROWSER_ID_DEV';
	var memberCookieName = isProd ? 'MEETUP_MEMBER' : 'MEETUP_MEMBER_DEV';

	var trackers = getTrackers({
		agent: agent,
		browserIdCookieName: browserIdCookieName,
		memberCookieName: memberCookieName,
		trackIdCookieName: trackIdCookieName
	});

	Object.keys(trackers).forEach(function (trackType) {
		server.decorate('request', trackType, trackers[trackType], {
			apply: true // make the `request` available to tracker
		});
	});

	server.ext('onRequest', onRequest);
	server.ext('onPreResponse', getOnPreResponse({
		browserIdCookieName: browserIdCookieName,
		memberCookieName: memberCookieName,
		trackIdCookieName: trackIdCookieName
	}));

	next();
}

register.attributes = {
	name: ACTIVITY_PLUGIN_NAME,
	version: '1.0.0'
};
//# sourceMappingURL=activity.js.map