'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _escapeHtml = require('escape-html');

var _escapeHtml2 = _interopRequireDefault(_escapeHtml);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getInnerHTML(__html) {
	return {
		__html: __html
	};
}

/**
 * Module that builds html, head, and body elements
 * And inserts app markup in the body
 *
 * Elements should only be added inside head via Helmet
 * @see {@link https://github.com/nfl/react-helmet}
 *
 * This component solves some "hard" problems related to wrapping a React
 * application in the full HTML markup needed to render it on the server.
 *
 * 1. Passing server-side app state to the client-side JS application - the
 * `window.INITIAL_STATE` setting, which has a few subtle gotchas if
 * implemented differently
 * 2. `<head>` management through `react-helmet` - there are lots of ways of
 * doing this wrong or implementing a fragile/inflexible solution
 * 3. Using an injected variable to write the client JS application bundle path
 * - there are lots of ways to introduce bugs into alternative implementations.
 *
 * @module DOM
 */
var DOM = function DOM(props) {
	var _props$appMarkup = props.appMarkup,
	    appMarkup = _props$appMarkup === undefined ? '' : _props$appMarkup,
	    assetPublicPath = props.assetPublicPath,
	    baseUrl = props.baseUrl,
	    head = props.head,
	    _props$initialState = props.initialState,
	    initialState = _props$initialState === undefined ? {} : _props$initialState,
	    scripts = props.scripts,
	    cssLinks = props.cssLinks;

	/**
  * `initialState` has untrusted user-generated content that needs to be
  * written into the DOM inside a <script> tag.
  *
  * It needs to be transformed into a plain object containing only escaped content
  */

	// stringify the whole object so that it can be escaped in one pass

	var initialStateJson = JSON.stringify(initialState);
	// escape the string
	var escapedState = (0, _escapeHtml2.default)(initialStateJson);

	var APP_RUNTIME = {
		baseUrl: baseUrl,
		assetPublicPath: assetPublicPath,
		escapedState: escapedState
	};

	return _react2.default.createElement(
		'html',
		null,
		_react2.default.createElement(
			'head',
			null,
			head.title.toComponent(),
			head.meta.toComponent(),
			head.link.toComponent(),
			head.script.toComponent(),
			cssLinks && cssLinks.map(function (href, key) {
				return _react2.default.createElement('link', { rel: 'stylesheet', type: 'text/css', href: href, key: key });
			})
		),
		_react2.default.createElement(
			'body',
			null,
			_react2.default.createElement('div', { id: 'outlet', dangerouslySetInnerHTML: getInnerHTML(appMarkup) }),
			_react2.default.createElement('script', {
				dangerouslySetInnerHTML: getInnerHTML('window.APP_RUNTIME=' + JSON.stringify(APP_RUNTIME) + ';')
			}),
			scripts.map(function (url, key) {
				return _react2.default.createElement('script', { src: url, key: key });
			})
		)
	);
};

DOM.propTypes = {
	appMarkup: _propTypes2.default.string,
	assetPublicPath: _propTypes2.default.string.isRequired,
	baseUrl: _propTypes2.default.string,
	head: _propTypes2.default.shape({
		// this is expected to come from Helmet.rewind()
		title: _propTypes2.default.shape({ toComponent: _propTypes2.default.func }),
		meta: _propTypes2.default.shape({ toComponent: _propTypes2.default.func }),
		link: _propTypes2.default.shape({ toComponent: _propTypes2.default.func }),
		script: _propTypes2.default.shape({ toComponent: _propTypes2.default.func })
	}),
	initialState: _propTypes2.default.object.isRequired,
	scripts: _propTypes2.default.array.isRequired,
	cssLinks: _propTypes2.default.arrayOf(_propTypes2.default.string)
};

exports.default = DOM;
//# sourceMappingURL=Dom.js.map