'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.config = exports.validateServerHost = exports.validateProtocol = exports.PROTOCOL_ERROR = exports.validatePhotoScalerSalt = exports.validateOauthKey = exports.validateOauthSecret = exports.validateCsrfSecret = exports.validateCookieSecret = exports.SALT_ERROR = exports.COOKIE_SECRET_ERROR = exports.OAUTH_KEY_ERROR = exports.OAUTH_SECRET_ERROR = exports.CSRF_SECRET_ERROR = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _convict = require('convict');

var _convict2 = _interopRequireDefault(_convict);

var _config = require('mwp-cli/src/config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var envSchema = _config.env.schema;
var envConfig = _config.env.properties;

/**
 * This module provides a single source of truth for application configuration
 * info. It uses node-convict to read configuration info from, in increasing
 * order of precedence:
 *
 * 1. Default value
 * 2. File (`config.loadFile()`) - app root `config.<NODE_ENV>.json`
 * 3. Environment variables
 * 4. Command line arguments
 * 5. Set and load calls (config.set() and config.load())
 *
 * Note that environment variables have _higher_ precedence than values in
 * config files, so the config files will only work if environment variables
 * are cleared.
 *
 * The only values that _must_ be in environment variables are the secrets that
 * are used to interact with external systems:
 *
 * - `MUPWEB_OAUTH_SECRET`
 * - `MUPWEB_OAUTH_KEY`
 * - `PHOTO_SCALER_SALT`
 *
 * @module config
 */

var random32 = 'asdfasdfasdfasdfasdfasdfasdfasdf';
var secretDefault = process.env.NODE_ENV !== 'production' && random32 || ''; // no prod default

var CSRF_SECRET_ERROR = exports.CSRF_SECRET_ERROR = 'CSRF Secret must be a random 32+ char string';
var OAUTH_SECRET_ERROR = exports.OAUTH_SECRET_ERROR = 'Invalid OAUTH Secret';
var OAUTH_KEY_ERROR = exports.OAUTH_KEY_ERROR = 'Invalid OAUTH Key';
var COOKIE_SECRET_ERROR = exports.COOKIE_SECRET_ERROR = 'Cookie Secret must be a random 32+ char string';
var SALT_ERROR = exports.SALT_ERROR = 'Invalid Photo Scaler Salt';

var validateCookieSecret = exports.validateCookieSecret = function validateCookieSecret(secret) {
	if (!secret || secret.toString().length < 32) {
		throw new Error(COOKIE_SECRET_ERROR);
	}
};

var validateCsrfSecret = exports.validateCsrfSecret = function validateCsrfSecret(secret) {
	if (!secret || secret.toString().length < 32) {
		throw new Error(CSRF_SECRET_ERROR);
	}
};

var validateOauthSecret = exports.validateOauthSecret = function validateOauthSecret(secret) {
	if (!secret || secret.toString().length < 1) {
		throw new Error(OAUTH_SECRET_ERROR);
	}
};

var validateOauthKey = exports.validateOauthKey = function validateOauthKey(key) {
	if (!key || key.toString().length < 1) {
		throw new Error(OAUTH_KEY_ERROR);
	}
};

var validatePhotoScalerSalt = exports.validatePhotoScalerSalt = function validatePhotoScalerSalt(salt) {
	if (!salt || salt.toString().length < 1) {
		throw new Error(SALT_ERROR);
	}
};

var PROTOCOL_ERROR = exports.PROTOCOL_ERROR = 'Protocol must be http or https';

var validateProtocol = exports.validateProtocol = function validateProtocol(protocol) {
	if (!['http', 'https'].includes(protocol)) {
		throw new Error(PROTOCOL_ERROR);
	}
};

var DEV_SUBSTRING = '.dev.';
var validateServerHost = exports.validateServerHost = function validateServerHost(host) {
	if (typeof host !== 'string') {
		throw new Error('Server host property must be a string');
	}
	var isDev = process.env.NODE_ENV === 'development' || !process.env.NODE_ENV;
	if (process.env.NODE_ENV === 'production' && host.includes(DEV_SUBSTRING)) {
		throw new Error('Server host ' + host + ' must not include \'.dev.\' in production');
	}
	if (isDev && !host.includes(DEV_SUBSTRING)) {
		throw new Error('Server host ' + host + ' must include \'.dev.\' in development');
	}
};

var config = exports.config = (0, _convict2.default)(_extends({}, envSchema, {
	app_server: {
		protocol: {
			format: validateProtocol,
			default: process.env.NODE_ENV === 'production' ? 'http' // SSL handled by load balancer
			: 'https',
			env: 'DEV_SERVER_PROTOCOL' // legacy naming
		},
		// host: '0.0.0.0', ALWAYS 0.0.0.0
		port: {
			format: 'port',
			default: 8000,
			arg: 'app-port',
			env: process.env.NODE_ENV !== 'test' && 'DEV_SERVER_PORT' // don't read env in tests
		},
		key_file: {
			format: String,
			default: _path2.default.resolve(_os2.default.homedir(), '.certs', 'star.dev.meetup.com.key'),
			env: 'APP_KEY_FILE'
		},
		crt_file: {
			format: String,
			default: _path2.default.resolve(_os2.default.homedir(), '.certs', 'star.dev.meetup.com.crt'),
			env: 'APP_CRT_FILE'
		}
	},
	api: {
		protocol: {
			format: validateProtocol,
			default: 'https',
			env: 'API_PROTOCOL'
		},
		host: {
			format: validateServerHost,
			default: 'api.dev.meetup.com',
			env: 'API_HOST'
		},
		timeout: {
			format: 'int',
			default: 10000,
			env: 'API_TIMEOUT'
		},
		root_url: {
			format: String,
			default: ''
		}
	},
	cookie_encrypt_secret: {
		format: validateCookieSecret,
		default: secretDefault,
		env: process.env.NODE_ENV !== 'test' && 'COOKIE_ENCRYPT_SECRET', // don't read env in tests
		sensitive: true
	},
	csrf_secret: {
		format: validateCsrfSecret,
		default: secretDefault,
		env: 'CSRF_SECRET',
		sensitive: true
	},
	oauth: {
		auth_url: {
			format: 'url',
			default: 'https://secure.dev.meetup.com/oauth2/authorize',
			env: 'OAUTH_AUTH_URL'
		},
		access_url: {
			format: 'url',
			default: 'https://secure.dev.meetup.com/oauth2/access',
			env: 'OAUTH_ACCESS_URL'
		},
		secret: {
			format: validateOauthSecret,
			default: null,
			env: 'MUPWEB_OAUTH_SECRET',
			sensitive: true
		},
		key: {
			format: validateOauthKey,
			default: null,
			env: 'MUPWEB_OAUTH_KEY',
			sensitive: true
		}
	},
	photo_scaler_salt: {
		format: validatePhotoScalerSalt,
		default: null,
		env: 'PHOTO_SCALER_SALT',
		sensitive: true
	}
}));

config.load(envConfig);

// Load environment dependent configuration
var configPath = _path2.default.resolve(process.cwd(), 'config.' + config.get('env') + '.json');

var localConfig = _fs2.default.existsSync(configPath) ? require(configPath) : {};

config.load(localConfig);

config.set('api.root_url', config.get('api.protocol') + '://' + config.get('api.host'));

config.set('isProd', config.get('env') === 'production');
config.set('isDev', config.get('env') === 'development');
var appConf = config.get('app_server');
if (appConf.protocol === 'https' && (!_fs2.default.existsSync(appConf.key_file) || !_fs2.default.existsSync(appConf.crt_file))) {
	var message = 'Missing HTTPS cert or key for application server!';
	if (config.isProd) {
		throw new Error(message);
	}
	console.error(_chalk2.default.red(message));
	console.warn(_chalk2.default.yellow('Re-setting protocol to HTTP - some features may not work as expected'));
	console.warn(_chalk2.default.yellow('See MWP config docs to configure HTTPS'));
	config.set('app_server.protocol', 'http');
}
config.validate();

exports.default = config.getProperties();
//# sourceMappingURL=index.js.map